"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.withInspector = exports.inspectorAgent = void 0;
var zlib_1 = __importDefault(require("zlib"));
var stream_1 = require("stream");
var undici_1 = require("undici");
var type_is_1 = __importDefault(require("type-is"));
var destroy_1 = __importDefault(require("destroy"));
var index_1 = require("./index");
var contant_1 = require("./contant");
var utils_1 = require("./utils");
exports.inspectorAgent = withInspector(new undici_1.Agent());
function withInspector(agent) {
    var dispatch = agent.dispatch;
    agent.dispatch = function (options, handler) {
        var _a;
        var reqHeaders = {};
        if (options.headers) {
            if (Array.isArray(options.headers)) {
                for (var n = 0; n < options.headers.length; n += 2) {
                    var key = options.headers[n + 0].toLowerCase();
                    var val = options.headers[n + 1].toLowerCase();
                    reqHeaders[key] = val;
                }
            }
            else {
                Object.assign(reqHeaders, options.headers);
            }
        }
        var req = {
            headers: reqHeaders,
            method: options.method,
            remoteAddress: '',
            time: Date.now(),
            type: options.body
                ? (type_is_1.default.is(String((_a = reqHeaders['content-type']) !== null && _a !== void 0 ? _a : ''), contant_1.CONTENT_TYPES) || undefined)
                : undefined,
            url: options.origin + options.path,
            rawBody: typeof options.body === 'string'
                ? Buffer.from(options.body)
                : Buffer.isBuffer(options.body)
                    ? options.body
                    : undefined,
            textBody: typeof options.body === 'string'
                ? options.body
                : Buffer.isBuffer(options.body)
                    ? options.body.toString('utf-8')
                    : undefined
        };
        var res = {
            status: 0,
            cost: 0,
            headers: {},
            type: undefined,
        };
        var input = new stream_1.PassThrough();
        var decoders = [];
        var onHeaders = handler.onHeaders;
        handler.onHeaders = function (status, headersList, resume) {
            var _a, _b, _c;
            res.status = status;
            if (headersList) {
                for (var n = 0; n < headersList.length; n += 2) {
                    var key = headersList[n + 0].toString('latin1').toLowerCase();
                    var val = headersList[n + 1].toString('latin1').toLowerCase();
                    res.headers[key] = val;
                }
            }
            res.type = (0, utils_1.hasbody)(res.headers)
                ? (type_is_1.default.is(String((_a = res.headers['content-type']) !== null && _a !== void 0 ? _a : ''), contant_1.CONTENT_TYPES) || undefined)
                : undefined;
            var encodings = String((_b = res.headers['content-encoding']) !== null && _b !== void 0 ? _b : '')
                .split(',')
                .map(function (val) { return val.trim(); })
                .filter(function (val) { return val.length; });
            for (var _i = 0, encodings_1 = encodings; _i < encodings_1.length; _i++) {
                var encoding = encodings_1[_i];
                switch (encoding) {
                    case 'gzip':
                    case 'x-gzip':
                    case 'compress':
                    case 'x-compress':
                    case 'deflate':
                        decoders.push(zlib_1.default.createUnzip(contant_1.zlibOptions));
                        break;
                    case 'br':
                        decoders.push(zlib_1.default.createBrotliDecompress(contant_1.brotliOptions));
                        break;
                }
            }
            var responseBody = [];
            for (var i = 0; i < decoders.length; i++) {
                ((_c = decoders[i - 1]) !== null && _c !== void 0 ? _c : input).pipe(decoders[i]);
            }
            var output = decoders.length ? decoders[decoders.length - 1] : input;
            output.on('data', function (chunk) {
                responseBody.push(chunk);
            });
            output.on('end', function () {
                res.cost = Date.now() - req.time;
                res.rawBody = Buffer.concat(responseBody);
                if (res.type) {
                    res.textBody = res.rawBody.toString('utf-8');
                }
                (0, destroy_1.default)(input);
                for (var _i = 0, decoders_1 = decoders; _i < decoders_1.length; _i++) {
                    var decoder = decoders_1[_i];
                    (0, destroy_1.default)(decoder);
                }
                (0, index_1.emit)({ type: 'undici', req: req, res: res });
            });
            output.on('error', function () {
                (0, destroy_1.default)(input);
                for (var _i = 0, decoders_2 = decoders; _i < decoders_2.length; _i++) {
                    var decoder = decoders_2[_i];
                    (0, destroy_1.default)(decoder);
                }
            });
            if (onHeaders) {
                return onHeaders.call(this, status, headersList, resume);
            }
            return true;
        };
        var onData = handler.onData;
        handler.onData = function (chunk) {
            input.push(chunk);
            if (onData) {
                return onData.call(this, chunk);
            }
            return true;
        };
        var onComplete = handler.onComplete;
        handler.onComplete = function (trailers) {
            input.end();
            if (onComplete) {
                return onComplete.call(this, trailers);
            }
        };
        return dispatch.call(this, options, handler);
    };
    return agent;
}
exports.withInspector = withInspector;
//# sourceMappingURL=undici.js.map